@@ -12,22 +12,26 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import fixtures
+import json
 import time
 
 import mock
+from oslo_config import cfg
 from oslo_config import fixture as fixture_config
+
 import oslo_messaging
-from oslo_serialization import jsonutils
-from oslotest import mockpatch
 import requests
 import six.moves.urllib.parse as urlparse
 
+from aodh import keystone_client
 from aodh import notifier
 from aodh import service
+
 from aodh.tests import base as tests_base
 
 
-DATA_JSON = jsonutils.loads(
+DATA_JSON = json.loads(
     '{"current": "ALARM", "alarm_id": "foobar", "alarm_name": "testalarm",'
     ' "severity": "critical", "reason": "what ?",'
     ' "reason_data": {"test": "test"}, "previous": "OK"}'
@@ -55,6 +59,52 @@ class TestAlarmNotifierService(tests_base.BaseTestCase):
         self.service.terminate()
 
 
+class TestKeystoneClient(tests_base.BaseTestCase):
+    def setUp(self):
+        super(TestKeystoneClient, self).setUp()
+        self.config = fixture_config.Config(
+            service.prepare_service(argv=[], config_files=[]))
+        self.config.setUp()
+        self.config.register_opts([
+            cfg.StrOpt('user_domain_id', default=""),
+            cfg.StrOpt('user_domain_name', default=""),
+            cfg.StrOpt('username', default="username"),
+            cfg.StrOpt('password', default="password"),
+            cfg.StrOpt('auth_url', default="testdomain")
+        ], "service_credentials")
+
+    def test_get_trusted_client_domain_id(self):
+        self.config.config(
+            **{'group': "service_credentials",
+               'user_domain_id': "uuid-domain"})
+
+        client = keystone_client.get_trusted_client(
+            self.config.conf, "testing")
+        self.assertEqual(client.session.auth._user_domain_id, "uuid-domain")
+        self.assertEqual(client.session.auth._user_domain_name, '')
+
+    def test_get_trusted_client_domain_name(self):
+        self.config.config(
+            **{'group': "service_credentials",
+               'user_domain_name': "testdomain"})
+
+        client = keystone_client.get_trusted_client(
+            self.config.conf, "testing")
+        self.assertEqual(client.session.auth._user_domain_name, "testdomain")
+        self.assertEqual(client.session.auth._user_domain_id, '')
+
+    def test_get_trusted_client_domain(self):
+        self.config.config(**{'group': "service_credentials",
+                              'user_domain_name': "testdomain",
+                              'user_domain_id': "uuid-gen",
+                              })
+
+        client = keystone_client.get_trusted_client(self.config.conf,
+                                                    "testing")
+        self.assertEqual(client.session.auth._user_domain_name, "testdomain")
+        self.assertEqual(client.session.auth._user_domain_id, "uuid-gen")
+
+
 class TestAlarmNotifier(tests_base.BaseTestCase):
     def setUp(self):
         super(TestAlarmNotifier, self).setUp()
@@ -65,7 +115,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
             self.transport, topics=['alarming'], driver='messaging',
             publisher_id='testpublisher')
         self.zaqar = FakeZaqarClient(self)
-        self.useFixture(mockpatch.Patch(
+        self.useFixture(fixtures.MockPatch(
             'aodh.notifier.zaqar.ZaqarAlarmNotifier.get_zaqar_client',
             return_value=self.zaqar))
         self.service = notifier.AlarmNotifierService(0, self.CONF)
@@ -175,7 +225,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     'content-type': 'application/json'
                 },
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
             self.assertEqual(2, len(m_log.info.call_args_list))
             expected = mock.call('Notifying alarm <%(id)s> gets response: '
                                  '%(status_code)s %(reason)s.',
@@ -204,7 +254,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     'content-type': 'application/json'
                 },
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_client_cert_and_key(self):
         action = 'https://host/action'
@@ -229,7 +279,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     kwargs['headers']['x-openstack-request-id'],
                     'content-type': 'application/json'},
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_disable_by_cfg(self):
         action = 'https://host/action'
@@ -252,7 +302,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     'content-type': 'application/json'
                 },
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_server_verify_enable(self):
         action = 'https://host/action'
@@ -270,7 +320,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                                       headers=mock.ANY,
                                       verify=ca_bundle)
             args, kwargs = poster.call_args
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_disable(self):
         action = 'https://host/action?aodh-alarm-ssl-verify=0'
@@ -291,7 +341,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     'content-type': 'application/json'
                 },
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_notify_alarm_rest_action_with_ssl_verify_enable_by_user(self):
         action = 'https://host/action?aodh-alarm-ssl-verify=1'
@@ -314,7 +364,7 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                     'content-type': 'application/json'
                 },
                 kwargs['headers'])
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     @staticmethod
     def _fake_urlsplit(*args, **kwargs):
@@ -356,8 +406,8 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
         client.session.auth.get_access.return_value.auth_token = 'token_1234'
 
         self.useFixture(
-            mockpatch.Patch('aodh.keystone_client.get_trusted_client',
-                            lambda *args: client))
+            fixtures.MockPatch('aodh.keystone_client.get_trusted_client',
+                               lambda *args: client))
 
         with mock.patch.object(requests.Session, 'post') as poster:
             self._msg_notifier.sample({}, 'alarm.update',
@@ -375,58 +425,77 @@ class TestAlarmNotifier(tests_base.BaseTestCase):
                 },
                 kwargs['headers'])
 
-            self.assertEqual(DATA_JSON, jsonutils.loads(kwargs['data']))
+            self.assertEqual(DATA_JSON, json.loads(kwargs['data']))
 
     def test_zaqar_notifier_action(self):
-        action = 'zaqar://?topic=critical&subscriber=http://example.com/data' \
-                 '&subscriber=mailto:foo@example.com&ttl=7200'
-        self._msg_notifier.sample({}, 'alarm.update',
-                                  self._notification(action))
-        time.sleep(1)
-        self.assertEqual(self.zaqar,
-                         self.service.notifiers['zaqar'].obj.client)
-
-    def test_presigned_zaqar_notifier_action(self):
         with mock.patch.object(notifier.zaqar.ZaqarAlarmNotifier,
-                               'get_presigned_client') as zaqar_client:
-            zaqar_client.return_value = self.zaqar, 'foobar-critical'
-            action = 'zaqar://?topic=critical&' \
-                     'subscriber=http://example.com/data' \
-                     '&subscriber=mailto:foo@example.com&ttl=7200' \
-                     '&signature=mysignature&expires=2016-06-29T01:49:56' \
-                     '&paths=/v2/queues/beijing/messages' \
-                     '&methods=GET,PATCH,POST,PUT&queue_name=foobar-critical' \
-                     '&project_id=my_project_id'
+                               '_get_client_conf') as get_conf:
+            action = ('zaqar://?topic=critical'
+                      '&subscriber=http://example.com/data'
+                      '&subscriber=mailto:foo@example.com&ttl=7200')
             self._msg_notifier.sample({}, 'alarm.update',
                                       self._notification(action))
             time.sleep(1)
-            self.assertEqual(zaqar_client.return_value[0],
-                             self.service.notifiers['zaqar'].obj.client)
-            queue_info = urlparse.parse_qs(urlparse.urlparse(action).query)
-            zaqar_client.assert_called_with(queue_info)
+            get_conf.assert_called()
+            self.assertEqual(self.zaqar,
+                             self.service.notifiers['zaqar'].obj._zclient)
+            self.assertEqual(2, self.zaqar.subscriptions)
+            self.assertEqual(1, self.zaqar.posts)
+
+    def test_presigned_zaqar_notifier_action(self):
+        action = ('zaqar://?'
+                  'subscriber=http://example.com/data&ttl=7200'
+                  '&signature=mysignature&expires=2016-06-29T01:49:56'
+                  '&paths=/v2/queues/beijing/messages'
+                  '&methods=GET,PATCH,POST,PUT&queue_name=foobar-critical'
+                  '&project_id=my_project_id')
+        self._msg_notifier.sample({}, 'alarm.update',
+                                  self._notification(action))
+        time.sleep(1)
+        self.assertEqual(1, self.zaqar.subscriptions)
+        self.assertEqual(1, self.zaqar.posts)
+
+    def test_trust_zaqar_notifier_action(self):
+        client = mock.MagicMock()
+        client.session.auth.get_access.return_value.auth_token = 'token_1234'
+
+        self.useFixture(
+            fixtures.MockPatch('aodh.keystone_client.get_trusted_client',
+                               lambda *args: client))
+
+        action = 'trust+zaqar://trust-1234:delete@?queue_name=foobar-critical'
+        self._msg_notifier.sample({}, 'alarm.update',
+                                  self._notification(action))
+        time.sleep(1)
+        self.assertEqual(0, self.zaqar.subscriptions)
+        self.assertEqual(1, self.zaqar.posts)
 
 
 class FakeZaqarClient(object):
 
     def __init__(self, testcase):
-        self.client = testcase
+        self.testcase = testcase
+        self.subscriptions = 0
+        self.posts = 0
 
     def queue(self, queue_name, **kwargs):
-        self.client.assertEqual('foobar-critical', queue_name)
-        self.client.assertEqual(dict(force_create=True), kwargs)
-        return FakeZaqarQueue(self.client)
+        self.testcase.assertEqual('foobar-critical', queue_name)
+        self.testcase.assertEqual({}, kwargs)
+        return FakeZaqarQueue(self)
 
     def subscription(self, queue_name, **kwargs):
-        self.client.assertEqual('foobar-critical', queue_name)
+        self.testcase.assertEqual('foobar-critical', queue_name)
         subscribers = ['http://example.com/data', 'mailto:foo@example.com']
-        self.client.assertIn(kwargs['subscriber'], subscribers)
-        self.client.assertEqual('7200', kwargs['ttl'])
+        self.testcase.assertIn(kwargs['subscriber'], subscribers)
+        self.testcase.assertEqual(7200, kwargs['ttl'])
+        self.subscriptions += 1
 
 
 class FakeZaqarQueue(object):
 
-    def __init__(self, testcase):
-        self.queue = testcase
+    def __init__(self, client):
+        self.client = client
+        self.testcase = client.testcase
 
     def post(self, message):
         expected_message = {'body': {'alarm_name': 'testalarm',
@@ -436,4 +505,5 @@ class FakeZaqarQueue(object):
                                      'reason': 'what ?',
                                      'severity': 'critical',
                                      'previous': 'OK'}}
-        self.queue.assertEqual(expected_message, message)
+        self.testcase.assertEqual(expected_message, message)
+        self.client.posts += 1
